@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -401,7 +401,8 @@ new_page_size(pcl_state_t * pcs,
      * an underline to be put out.
      */
     pcs->underline_enabled = false;
-    pcl_home_cursor(pcs);
+    code = pcl_home_cursor(pcs);
+    if (code < 0) return code;
     /*
      * this is were we initialized the cursor position
      */
@@ -602,9 +603,9 @@ pcl_cursor_moved(pcl_state_t * pcs)
 int
 pcl_end_page(pcl_state_t * pcs, pcl_print_condition_t condition)
 {
-    int code = 0;
-
-    pcl_break_underline(pcs);   /* (could mark page) */
+    int code = pcl_break_underline(pcs);   /* (could mark page) */
+    if (code < 0)
+        return code;
 
     /* If we are conditionally printing (normal case) check if the
        page is marked */
@@ -694,7 +695,9 @@ set_page_size(pcl_args_t * pargs, pcl_state_t * pcs)
     code = pcl_end_page_if_marked(pcs);
     if (code < 0)
         return code;
-    pcl_home_cursor(pcs);
+    code = pcl_home_cursor(pcs);
+    if (code < 0)
+        return code;
 
     for (i = 0; i < pcl_paper_type_count; i++) {
         if (tag == PAPER_SIZES[i].tag) {
@@ -724,7 +727,9 @@ set_paper_source(pcl_args_t * pargs, pcl_state_t * pcs)
 
     if (code < 0)
         return code;
-    pcl_home_cursor(pcs);
+    code = pcl_home_cursor(pcs);
+    if (code < 0)
+        return code;
     /* Do not change the page side if the wanted paper source is the same as the actual one */
     if (pcs->paper_source != i) {
         pcs->back_side = false;
@@ -800,7 +805,11 @@ set_logical_page_orientation(pcl_args_t * pargs, pcl_state_t * pcs)
        set the flag disabling the orientation command for this page. */
     code = pcl_end_page_if_marked(pcs);
     if (code >= 0) {
+        /* a page_orientation change is not the same as a new page.
+           If a cursor has moved this should be remembered */
+        bool cursor_moved = pcs->cursor_moved;
         code = new_logical_page(pcs, i, pcs->xfm_state.paper_size, false, false);
+        pcs->cursor_moved = cursor_moved;
         pcs->hmi_cp = HMI_DEFAULT;
         pcs->vmi_cp = VMI_DEFAULT;
     }
@@ -818,7 +827,9 @@ set_print_direction(pcl_args_t * pargs, pcl_state_t * pcs)
     if ((i <= 270) && (i % 90 == 0)) {
         i /= 90;
         if (i != pcs->xfm_state.print_dir) {
-            pcl_break_underline(pcs);
+            int code = pcl_break_underline(pcs);
+            if (code < 0)
+                return code;
             pcs->xfm_state.print_dir = i;
             update_xfm_state(pcs, 0);
             pcl_continue_underline(pcs);
@@ -838,7 +849,11 @@ set_print_direction(pcl_args_t * pargs, pcl_state_t * pcs)
 static int
 set_left_margin(pcl_args_t * pargs, pcl_state_t * pcs)
 {
-    coord lmarg = uint_arg(pargs) * pcl_hmi(pcs);
+    int code = pcl_update_hmi_cp(pcs);
+    coord lmarg = uint_arg(pargs) * pcs->hmi_cp;
+
+    if (code < 0)
+        return code;
 
     /* adjust underlining if the left margin passes to the right of
        the underline start position */
@@ -848,9 +863,9 @@ set_left_margin(pcl_args_t * pargs, pcl_state_t * pcs)
     if (lmarg < pcs->margins.right) {
         pcs->margins.left = lmarg;
         if (pcs->cap.x < lmarg)
-            pcl_set_cap_x(pcs, lmarg, false, false);
+            code = pcl_set_cap_x(pcs, lmarg, false, false);
     }
-    return 0;
+    return code;
 }
 
 /*
@@ -862,17 +877,21 @@ set_left_margin(pcl_args_t * pargs, pcl_state_t * pcs)
 static int
 set_right_margin(pcl_args_t * pargs, pcl_state_t * pcs)
 {
-    coord rmarg = (uint_arg(pargs) + 1) * pcl_hmi(pcs);
+    int code = pcl_update_hmi_cp(pcs);
+    coord rmarg = (uint_arg(pargs) + 1) * pcs->hmi_cp;
+
+    if (code < 0)
+        return code;
 
     if (rmarg > pcs->xfm_state.pd_size.x)
         rmarg = pcs->xfm_state.pd_size.x;
     if (rmarg > pcs->margins.left) {
         pcs->margins.right = rmarg;
         if (pcs->cap.x > rmarg)
-            pcl_set_cap_x(pcs, rmarg, false, false);
+            code = pcl_set_cap_x(pcs, rmarg, false, false);
     }
 
-    return 0;
+    return code;
 }
 
 /*
@@ -968,7 +987,7 @@ pcl_media_type(pcl_args_t * pargs, pcl_state_t * pcs)
         int code = pcl_end_page_if_marked(pcs);
 
         if (code >= 0)
-            pcl_home_cursor(pcs);
+            code = pcl_home_cursor(pcs);
         return (code < 0 ? code : e_Unimplemented);
     } else
         return e_Range;
@@ -1135,7 +1154,7 @@ pcl_print_quality(pcl_args_t * pargs, pcl_state_t * pcs)
         int code = pcl_end_page_if_marked(pcs);
 
         if (code >= 0)
-            pcl_home_cursor(pcs);
+            code = pcl_home_cursor(pcs);
         return (code < 0 ? code : 0);
     } else
         return e_Range;
