@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -65,13 +65,6 @@ pcl_downloaded_and_bound(const pl_font_t * plfont)
     return (plfont->storage != pcds_internal && pl_font_is_bound(plfont));
 }
 
-/* uncomment the following definition to treat map type 0 as defined
-   in the specification.  The default is to use the behavior we have
-   observed on several HP devices.  Map type 0 is treated as map type
-   1. */
-
-/* #define USE_MAP_TYPE_IN_SPECIFICATION */
-
 /*
  * Check if a character code is considered "printable" by given symbol set.
  */
@@ -83,43 +76,44 @@ char_is_printable(const pl_font_t *font, const pl_symbol_map_t *map, gs_char chr
     if (literal) {              /* transparent data */
         printable = true;
     } else {
-    if (is_stick) {
-        printable = (chr >= ' ') && (chr <= 0xff);
-    } else {
-        int map_type = 0;
-        if (map == 0 || pcl_downloaded_and_bound(font)) {
-        /* PCL TRM 11-18 */
-            if (font)
-            {
-            map_type = font->font_type;
-        }
+        if (is_stick) {
+            printable = (chr >= ' ') && (chr <= 0xff);
         } else {
-        /* PCL TRM 10-7
-         * symbol map type overrides, font map type
-         */
-        map_type = map->type;
-        }
+            int map_type = 0;
+            if (map == 0 || pcl_downloaded_and_bound(font)) {
+                /* PCL TRM 11-18 */
+                if (font)
+                {
+                    map_type = font->font_type;
+                }
+            } else {
+                /* PCL TRM 10-7
+                 * symbol map type overrides, font map type
+                 */
+                map_type = map->type;
+            }
 
-#ifndef USE_MAP_TYPE_IN_SPECIFICATION
-        if (map_type == 0)
-        map_type = 1;
-#endif /* USE_MAP_TYPE_IN_SPECIFICATION */
+            /* We do not treat map type as defined in the
+            specification. Instead the default is to use the behavior we have
+            observed on several HP devices: Map type 0 is treated as map type
+            1. */
+            if (map_type == 0) {
+                map_type = 1;
+            }
 
-        if (map_type == 0)
-        printable = (chr >= ' ') && (chr <= '\177');
-        else if (map_type == 1) {
-        chr &= 0x7f;
-        printable = (chr >= ' ');   /* 0-31 and 128-159 are not printable */
-        } else if (map_type >= 2) {
-        /* 2 is correct but will force all types above 2 here */
-        if ((chr == 0) || (chr == '\033') ||
-            ((chr >= '\007') && (chr <= '\017')))
-            printable = false;
-        else
-            printable = true;
+            if (map_type == 1) {
+                chr &= 0x7f;
+                printable = (chr >= ' ');   /* 0-31 and 128-159 are not printable */
+            } else if (map_type >= 2) {
+                /* 2 is correct but will force all types above 2 here */
+                if ((chr == 0) || (chr == '\033') ||
+                    ((chr >= '\007') && (chr <= '\017')))
+                    printable = false;
+                else
+                    printable = true;
+            }
         }
     }
-    }
     return printable;
 }
 
@@ -503,11 +497,10 @@ show_char_foreground(const pcl_state_t * pcs, const gs_char * pbuff)
     text.data.chars = pbuff;
     text.size = 1;
     code = gs_text_begin(pcs->pgs, &text, pcs->memory, &penum);
-    if (code >= 0)
+    if (code >= 0) {
         code = gs_text_process(penum);
-
-    if (code >= 0)
         gs_text_release(penum, "show_char_foreground");
+    }
     return code;
 }
 
@@ -619,8 +612,10 @@ show_char_background(pcl_state_t * pcs, const gs_char * pbuff)
         return code;
     if (pcs->pattern_transparent) {
         code = pcl_set_drawing_color(pcs, pcl_pattern_solid_white, 0, false);
-        if (code < 0)
+        if (code < 0) {
+            (void)pcl_grestore(pcs);
             return code;
+        }
     }
     if (((code = gs_setrasterop(pgs, (gs_rop3_t) rop3_know_S_1((int)rop))) < 0) ||
         ((code = gs_currentpoint(pgs, &pt)) < 0)) {
@@ -646,7 +641,7 @@ show_char_background(pcl_state_t * pcs, const gs_char * pbuff)
             gs_image_enum_alloc(gs_gstate_memory(pgs),
                                 "bitmap font background");
         if (pen == 0) {
-            pcl_grestore(pcs);
+            (void)pcl_grestore(pcs);
             return e_Memory;
         }
 
@@ -671,11 +666,12 @@ show_char_background(pcl_state_t * pcs, const gs_char * pbuff)
     } else {
         gs_text_params_t text;
         gs_rect bbox;
-        gs_text_enum_t *penum;
+        gs_text_enum_t *penum = NULL;
 
         /* clear the path; start the new one from the current point */
         if (((code = gs_newpath(pgs)) < 0) ||
             ((code = gs_moveto(pgs, pt.x, pt.y)) < 0)) {
+            (void)pcl_grestore(pcs);
             return code;
         }
         text.data.chars = pbuff;
@@ -691,10 +687,13 @@ show_char_background(pcl_state_t * pcs, const gs_char * pbuff)
                 (code = gs_rectappend(pgs, &bbox, 1)) >= 0 &&
                 (code = gs_eofill(pgs)) >= 0)
             {
-                gs_text_release(penum, "show_char_background");
+                /* fall through */
             }
-            else
-                return code;
+        }
+        gs_text_release(penum, "show_char_background");
+        if (code < 0) {
+            (void)pcl_grestore(pcs);
+            return code;
         }
     }
 
@@ -704,17 +703,24 @@ show_char_background(pcl_state_t * pcs, const gs_char * pbuff)
 /*
  * get the advance width.
  */
-static double
+static int
 pcl_get_width(pcl_state_t * pcs, gs_point * advance_vector,
               const gs_point * pscale, gs_char chr, bool is_space,
-              bool print_undefined)
+              bool print_undefined, double *output_width)
 {
+    int code = 0;
     pcl_font_selection_t *pfp = &(pcs->font_selection[pcs->font_selected]);
     double width;
 
     if (chr != 0xffff || print_undefined) {
         if (!pfp->params.proportional_spacing || is_space)
-            width = pcl_hmi(pcs);
+        {
+            code = pcl_update_hmi_cp(pcs);
+            if (code < 0)
+                return code;
+
+            width = pcs->hmi_cp;
+        }
         else {
             if (pcs->font->scaling_technology == plfst_TrueType ||
                 pcs->font->scaling_technology == plfst_MicroType) {
@@ -730,12 +736,17 @@ pcl_get_width(pcl_state_t * pcs, gs_point * advance_vector,
             width += (double) pcs->uom_cp / 2.0;
             width -= fmod(width, (double) pcs->uom_cp);
         }
-    } else if (is_space)
-        width = pcl_hmi(pcs);
+    } else if (is_space) {
+        code = pcl_update_hmi_cp(pcs);
+        if (code < 0)
+            return code;
+        width = pcs->hmi_cp;
+    }
     else
         width = 0.0;
     /* round to nearest integral pcl units */
-    return width;
+    *output_width = width;
+    return code;
 }
 
 /*
@@ -828,9 +839,11 @@ pcl_show_chars_slow(pcl_state_t * pcs,
         buff[0] = chr;
 
         /* round width to integral pcl current units */
-        width =
+        code =
             (pcl_get_width
-             (pcs, &advance_vector, pscale, chr, is_space, print_undefined));
+             (pcs, &advance_vector, pscale, chr, is_space, print_undefined, &width));
+        if (code < 0)
+            return code;
 
         /*
          * Check for transitions of the left margin; this check is
@@ -861,8 +874,12 @@ pcl_show_chars_slow(pcl_state_t * pcs,
                        function. */
                     pcs->cap.x = (coord) cpt.x;
                     pcs->cap.y = (coord) cpt.y;
-                    pcl_do_CR(pcs);
-                    pcl_do_LF(pcs);
+                    code = pcl_do_CR(pcs);
+                    if (code < 0)
+                        return code;
+                    code = pcl_do_LF(pcs);
+                    if (code < 0)
+                        return code;
                     cpt.x = pcs->cap.x;
                     cpt.y = pcs->cap.y;
                     use_rmargin = true;
@@ -967,8 +984,8 @@ pcl_font_scale(pcl_state_t * pcs, gs_point * pscale)
 {
     /* set up the font transformation */
     if (pcs->font->scaling_technology == plfst_bitmap) {
-        pscale->x = pcl_coord_scale / pcs->font->resolution.x;
-        pscale->y = pcl_coord_scale / pcs->font->resolution.y;
+        pscale->x = (double) pcl_coord_scale / pcs->font->resolution.x;
+        pscale->y = (double) pcl_coord_scale / pcs->font->resolution.y;
     } else {
         /*
          * Outline fonts are 1-point; the font height is given in
@@ -1086,29 +1103,46 @@ pcl_plain_char(pcl_args_t * pargs, pcl_state_t * pcs)
                     pcs->display_functions);
 }
 
+/*
+ * Do any underlining just before a break in motion (vertical motion or
+ * negative horizontal motion)...
+ */
+int pcl_break_underline(pcl_state_t * pcs)
+{
+    int code = 0;
+
+    if (pcs->underline_enabled)
+        code = pcl_do_underline(pcs);
+
+    return code;
+}
+
 /*
  * draw underline up to current point, adjust status
  */
-void
+int
 pcl_do_underline(pcl_state_t * pcs)
 {
+    int code = 0;
+
     if (pcs->underline_start.x != pcs->cap.x) {
         gs_gstate *pgs = pcs->pgs;
         float y = pcs->underline_start.y + pcs->underline_position;
-        int code;
 
-        /* save the grapics state */
+        /* save the graphics state */
         code = pcl_gsave(pcs);
         if (code < 0)
-            return;
+            return code;
 
         code = pcl_set_drawing_color(pcs,
                                      pcs->pattern_type,
                                      pcs->current_pattern_id, false);
         if (code >= 0)
             code = pcl_set_graphics_state(pcs);
-        if (code < 0)
-            return;
+        if (code < 0) {
+            (void)pcl_grestore(pcs);
+            return code;
+        }
 
         /*
          * TRM says (8-34) that underline is 3 dots.  In a victory for
@@ -1119,10 +1153,13 @@ pcl_do_underline(pcl_state_t * pcs)
         if ((gs_moveto(pgs, pcs->underline_start.x, y) < 0) ||
             (gs_lineto(pgs, pcs->cap.x, y) < 0) ||
             (gs_stroke(pgs) < 0)) {
-            return;
+            (void)pcl_grestore(pcs);
+            return code;
         }
 
-        pcl_grestore(pcs);
+        code = pcl_grestore(pcs);
+        if (code < 0)
+            return code;
     }
 
     /*
@@ -1131,6 +1168,7 @@ pcl_do_underline(pcl_state_t * pcs)
      */
     pcs->underline_start = pcs->cap;
     pcs->underline_position = pcs->underline_floating ? 0.0 : dots(5);
+    return code;
 }
 
 /* ------ Commands ------ */
@@ -1192,14 +1230,16 @@ pcl_enable_underline(pcl_args_t * pargs, pcl_state_t * pcs)
 static int
 pcl_disable_underline(pcl_args_t * pargs, pcl_state_t * pcs)
 {
+    int code = 0;
+
     /* apparently disabling underlining has the side effect of
        flushing any pending underlines.  This side effect is not
        documented */
     if (pcs->underline_enabled == true) {
-        pcl_do_underline(pcs);
+        code = pcl_do_underline(pcs);
         pcs->underline_enabled = false;
     }
-    return 0;
+    return code;
 }
 
 /* (From PCL5 Comparison Guide, p. 1-56) */
