@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -975,7 +975,7 @@ process_ccitt_compress(gs_gstate * pgs,
         state.K = -1;
     s_CFD_template.init((stream_state *) & state);
     scr.ptr = pin + 4 - 1;
-    scr.limit = scr.ptr + insize;
+    scr.limit = scr.ptr + insize - 4;
 
     wrsize = (state.Columns + 7) / 8;
     temp_buffer = gs_alloc_bytes(prast->pmem, wrsize, "CCITT temp_buffer");
@@ -994,7 +994,11 @@ process_ccitt_compress(gs_gstate * pgs,
 
             case 1:            /* need output, process the scanline and continue. */
                 memcpy(pout->pdata, temp_buffer, min(pout->size, wrsize));
-                process_row(prast, 0);
+                code = process_row(prast, 0);
+                if (code < 0) {
+                    s_CFD_template.release((stream_state *) & state);
+                    return gs_rethrow(code, "CCITT decompression failed\n");
+                }
                 memset(temp_buffer, 0, wrsize);
                 scw.ptr = temp_buffer - 1;
                 scw.limit = scw.ptr + wrsize;
@@ -1023,7 +1027,7 @@ process_ccitt_compress(gs_gstate * pgs,
  */
 static int
 process_adaptive_compress(gs_gstate * pgs,
-                          pcl_raster_t * prast, const byte * pin, uint insize)
+                          pcl_raster_t * prast, const byte * pin, uint insize, bool mono)
 {
     pcl_seed_row_t *pseed_row = prast->pseed_rows;
     byte *pdata = pseed_row->pdata;
@@ -1088,8 +1092,10 @@ process_adaptive_compress(gs_gstate * pgs,
             break;
     }
 
-    /* clear the seed rows if Delta Row Compression at end of raster block */
-    if (cmd == DELTA_ROW_COMPRESS) {
+    /* On monochrome printers tested HP clears the seed row of delta
+       row compression after each block, color printers leave it
+       intact. */
+    if (mono) {
         memset(pdata, 0, row_size);
         pseed_row->is_blank = true;
     }
@@ -1151,7 +1157,7 @@ add_raster_plane(const byte * pdata,
         else if (comp_mode == NO_COMPRESS_BLOCK)
             return process_block_nocompress(pcs->pgs, prast, pdata, nbytes);
         else if (comp_mode == ADAPTIVE_COMPRESS)
-            return process_adaptive_compress(pcs->pgs, prast, pdata, nbytes);
+            return process_adaptive_compress(pcs->pgs, prast, pdata, nbytes, pcs->personality == pcl5e);
         else
             return process_ccitt_compress(pcs->pgs, prast, pdata, nbytes,
                                           comp_mode);
@@ -1437,6 +1443,9 @@ transfer_raster_row(pcl_args_t * pargs, pcl_state_t * pcs)
             process_row((pcl_raster_t *) pcs->raster_state.pcur_raster,
                         comp_mode);
 
+    if (code < 0)
+        (void)pcl_grestore(pcs);
+
     return code;
 }
 
